﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DebugEngine {

  [StorableClass]
  public class OperatorTrace : ObservableList<IOperator>, IContent, IDeepCloneable {

    public OperatorTrace()
      : base() {
    }
    public OperatorTrace(int capacity)
      : base(capacity) {
    }
    public OperatorTrace(IEnumerable<IOperator> collection)
      : base(collection) {
    }

    [StorableConstructor]
    protected OperatorTrace(bool deserializing) : base(deserializing) { }
    protected OperatorTrace(OperatorTrace original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      AddRange(original.Select(op => cloner.Clone(op)));
    }
    public object Clone() {
      return Clone(new Cloner());
    }
    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new OperatorTrace(this, cloner);
    }
    public virtual void ReplaceAll(IEnumerable<IOperator> operators) {
      var oldList = list;
      list = new List<IOperator>(operators);
      if (oldList.Count != list.Count)
        OnPropertyChanged("Count");
      OnPropertyChanged("Item[]");
      OnCollectionReset(
        list.Select((op, i) => new IndexedItem<IOperator>(i, op)),
        oldList.Select((op, i) => new IndexedItem<IOperator>(i, op)));
    }

  }
}
